@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
- |        PDFlib - A library for dynamically generating PDF documents        |
+ |              PDFlib - A library for generating PDF on the fly             |
  +---------------------------------------------------------------------------+
- |        Copyright (c) 1997-2000 Thomas Merz. All rights reserved.          |
+ | Copyright (c) 1997-2001 PDFlib GmbH and Thomas Merz. All rights reserved. |
  +---------------------------------------------------------------------------+
  |    This software is NOT in the public domain.  It can be used under two   |
  |    substantially different licensing terms:                               |
@@ -27,7 +27,7 @@
  |    however, will receive warranty and support statements in writing.      |
  *---------------------------------------------------------------------------*/
 
-/* pdflib_pl.c
+/* $Id: pdflib_pl.c,v 1.9 2001/04/09 19:55:38 tm Exp $
  *
  * Wrapper code for the PDFlib Perl binding
  *
@@ -47,6 +47,7 @@
 #ifdef __cplusplus
 #include <math.h>
 #include <stdlib.h>
+
 extern "C" {
 #endif /* __cplusplus */
 
@@ -61,6 +62,13 @@ extern "C" {
 }
 #endif /* __cplusplus */
 
+/* Compilers which are not strictly ANSI conforming can set PDF_VOLATILE
+ * to an empty value.
+ */
+#ifndef PDF_VOLATILE
+#define PDF_VOLATILE    volatile
+#endif
+
 /* Definitions for compiling Perl extensions on a variety of machines */
 
 #if defined(WIN32) || defined(__WIN32__)
@@ -115,14 +123,14 @@ extern "C" {
 
 typedef struct SwigPtrType {
   char               *name;
-  int                 len;
+  int               len;
   void               *(*cast)(void *);
   struct SwigPtrType *next;
 } SwigPtrType;
 
 /* Pointer cache structure */
 typedef struct {
-  int                 stat;               /* Status (valid) bit             */
+  int               stat;               /* Status (valid) bit             */
   SwigPtrType        *tp;                 /* Pointer to type structure      */
   char                name[256];          /* Given datatype name            */
   char                mapped[256];        /* Equivalent name                */
@@ -219,8 +227,8 @@ char *_SWIG_GetPtr(CPerlObj *pPerl, SV *sv, void **ptr, char *_t)
 {
   char temp_type[256];
   char *name,*_c;
-  int  len,i,start,end;
-  IV   tmp;
+  int len,i,start,end;
+  IV   tmp = 0;
   SwigPtrType *sp,*tp;
   SwigCacheType *cache;
 
@@ -404,8 +412,7 @@ pdf_swig_errorhandler(PDF *p, int type, const char* shortmsg)
     }
 	
     /* Clean up PDFlib internals and delete p in all other cases */
-    if (p != NULL)
-	PDF_delete(p);
+    PDF_delete(p);
 
     /* ...and throw an exception */
     throw(type);
@@ -418,7 +425,7 @@ pdf_swig_errorhandler(PDF *p, int type, const char* shortmsg)
 
 XS(_wrap_PDF_new) {
     PDF * _result;
-    int argvi = 0;
+    int PDF_VOLATILE argvi = 0;
     dXSARGS ;
     char versionbuf[32];
 
@@ -468,10 +475,10 @@ XS(_wrap_PDF_delete) {
 }
 
 XS(_wrap_PDF_open_file) {
-    int  _result;
+    int _result;
     PDF * _arg0;
-    char * _arg1;
-    int argvi = 0;
+    char * PDF_VOLATILE _arg1;
+    int PDF_VOLATILE argvi = 0;
     dXSARGS ;
 
     if (items != 2)
@@ -538,8 +545,8 @@ XS(_wrap_PDF_get_buffer) {
 
 XS(_wrap_PDF_begin_page) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
+    float _arg1;
+    float _arg2;
     dXSARGS ;
 
     if (items != 3)
@@ -580,8 +587,8 @@ XS(_wrap_PDF_end_page) {
 
 XS(_wrap_PDF_set_parameter) {
     PDF * _arg0;
-    char * _arg1;
-    char * _arg2;
+    char * PDF_VOLATILE _arg1;
+    char * PDF_VOLATILE _arg2;
     dXSARGS ;
 
     if (items != 3)
@@ -604,9 +611,9 @@ XS(_wrap_PDF_set_parameter) {
 XS(_wrap_PDF_get_parameter) {
     char * _result;
     PDF * _arg0;
-    char * _arg1;
-    float  _arg2;
-    int argvi = 0;
+    char * PDF_VOLATILE _arg1;
+    float _arg2;
+    int PDF_VOLATILE argvi = 0;
     dXSARGS ;
 
     if (items != 3)
@@ -632,8 +639,8 @@ XS(_wrap_PDF_get_parameter) {
 
 XS(_wrap_PDF_set_value) {
     PDF * _arg0;
-    char * _arg1;
-    float  _arg2;
+    char * PDF_VOLATILE _arg1;
+    float _arg2;
     int argvi = 0;
     dXSARGS ;
 
@@ -656,11 +663,11 @@ XS(_wrap_PDF_set_value) {
 }
 
 XS(_wrap_PDF_get_value) {
-    float  _result;
+    float _result;
     PDF * _arg0;
-    char * _arg1;
-    float  _arg2;
-    int argvi = 0;
+    char * PDF_VOLATILE _arg1;
+    float _arg2;
+    int PDF_VOLATILE argvi = 0;
     dXSARGS ;
 
     if (items != 3)
@@ -685,12 +692,12 @@ XS(_wrap_PDF_get_value) {
 }
 
 XS(_wrap_PDF_findfont) {
-    int  _result;
+    int _result;
     PDF * _arg0;
-    char * _arg1;
-    char * _arg2;
-    int  _arg3;
-    int argvi = 0;
+    char * PDF_VOLATILE _arg1;
+    char * PDF_VOLATILE _arg2;
+    int PDF_VOLATILE _arg3;
+    int PDF_VOLATILE argvi = 0;
     dXSARGS ;
 
     if (items != 4)
@@ -715,8 +722,8 @@ XS(_wrap_PDF_findfont) {
 
 XS(_wrap_PDF_setfont) {
     PDF * _arg0;
-    int  _arg1;
-    float  _arg2;
+    int PDF_VOLATILE _arg1;
+    float _arg2;
     dXSARGS ;
 
     if (items != 3)
@@ -738,7 +745,7 @@ XS(_wrap_PDF_setfont) {
 
 XS(_wrap_PDF_show) {
     PDF * _arg0;
-    char * _arg1;
+    char * PDF_VOLATILE _arg1;
     dXSARGS ;
     size_t len;
 
@@ -760,9 +767,9 @@ XS(_wrap_PDF_show) {
 
 XS(_wrap_PDF_show_xy) {
     PDF * _arg0;
-    char * _arg1;
-    float  _arg2;
-    float  _arg3;
+    char * PDF_VOLATILE _arg1;
+    float _arg2;
+    float _arg3;
     size_t len;
     dXSARGS ;
 
@@ -786,7 +793,7 @@ XS(_wrap_PDF_show_xy) {
 
 XS(_wrap_PDF_continue_text) {
     PDF * _arg0;
-    char * _arg1;
+    char * PDF_VOLATILE _arg1;
     size_t len;
     dXSARGS ;
 
@@ -807,16 +814,16 @@ XS(_wrap_PDF_continue_text) {
 }
 
 XS(_wrap_PDF_show_boxed) {
-    int  _result;
+    int _result;
     PDF * _arg0;
-    char * _arg1;
-    float  _arg2;
-    float  _arg3;
-    float  _arg4;
-    float  _arg5;
-    char * _arg6;
-    char * _arg7;
-    int argvi = 0;
+    char * PDF_VOLATILE _arg1;
+    float _arg2;
+    float _arg3;
+    float _arg4;
+    float _arg5;
+    char * PDF_VOLATILE _arg6;
+    char * PDF_VOLATILE _arg7;
+    int PDF_VOLATILE argvi = 0;
     dXSARGS ;
 
     if (items != 8)
@@ -847,8 +854,8 @@ XS(_wrap_PDF_show_boxed) {
 
 XS(_wrap_PDF_set_text_pos) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
+    float _arg1;
+    float _arg2;
     dXSARGS ;
 
     if (items != 3)
@@ -870,12 +877,12 @@ XS(_wrap_PDF_set_text_pos) {
 
 XS(_wrap_PDF_set_text_matrix) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
-    float  _arg3;
-    float  _arg4;
-    float  _arg5;
-    float  _arg6;
+    float _arg1;
+    float _arg2;
+    float _arg3;
+    float _arg4;
+    float _arg5;
+    float _arg6;
     dXSARGS ;
 
     if (items != 7)
@@ -901,12 +908,12 @@ XS(_wrap_PDF_set_text_matrix) {
 }
 
 XS(_wrap_PDF_stringwidth) {
-    float  _result;
+    float _result;
     PDF * _arg0;
-    char * _arg1;
-    int  _arg2;
-    float  _arg3;
-    int argvi = 0;
+    char * PDF_VOLATILE _arg1;
+    int PDF_VOLATILE _arg2;
+    float _arg3;
+    int PDF_VOLATILE argvi = 0;
     size_t len;
     dXSARGS ;
 
@@ -932,8 +939,8 @@ XS(_wrap_PDF_stringwidth) {
 
 XS(_wrap_PDF_setdash) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
+    float _arg1;
+    float _arg2;
     dXSARGS ;
 
     if (items != 3)
@@ -958,7 +965,7 @@ XS(_wrap_PDF_setpolydash) {
     SV *val;
     AV *av;
     float carray[MAX_DASH_LENGTH];
-    int  length;
+    int PDF_VOLATILE length;
     int i;
     dXSARGS ;
 
@@ -1000,7 +1007,7 @@ XS(_wrap_PDF_setpolydash) {
 
 XS(_wrap_PDF_setflat) {
     PDF * _arg0;
-    float  _arg1;
+    float _arg1;
     dXSARGS ;
 
     if (items != 2)
@@ -1021,7 +1028,7 @@ XS(_wrap_PDF_setflat) {
 
 XS(_wrap_PDF_setlinejoin) {
     PDF * _arg0;
-    int  _arg1;
+    int PDF_VOLATILE _arg1;
     dXSARGS ;
 
     if (items != 2)
@@ -1042,7 +1049,7 @@ XS(_wrap_PDF_setlinejoin) {
 
 XS(_wrap_PDF_setlinecap) {
     PDF * _arg0;
-    int  _arg1;
+    int PDF_VOLATILE _arg1;
     dXSARGS ;
 
     if (items != 2)
@@ -1063,7 +1070,7 @@ XS(_wrap_PDF_setlinecap) {
 
 XS(_wrap_PDF_setmiterlimit) {
     PDF * _arg0;
-    float  _arg1;
+    float _arg1;
     dXSARGS ;
 
     if (items != 2)
@@ -1084,7 +1091,7 @@ XS(_wrap_PDF_setmiterlimit) {
 
 XS(_wrap_PDF_setlinewidth) {
     PDF * _arg0;
-    float  _arg1;
+    float _arg1;
     dXSARGS ;
 
     if (items != 2)
@@ -1143,8 +1150,8 @@ XS(_wrap_PDF_restore) {
 
 XS(_wrap_PDF_translate) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
+    float _arg1;
+    float _arg2;
     dXSARGS ;
 
     if (items != 3)
@@ -1166,8 +1173,8 @@ XS(_wrap_PDF_translate) {
 
 XS(_wrap_PDF_scale) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
+    float _arg1;
+    float _arg2;
     dXSARGS ;
 
     if (items != 3)
@@ -1189,7 +1196,7 @@ XS(_wrap_PDF_scale) {
 
 XS(_wrap_PDF_rotate) {
     PDF * _arg0;
-    float  _arg1;
+    float _arg1;
     dXSARGS ;
 
     if (items != 2)
@@ -1210,8 +1217,8 @@ XS(_wrap_PDF_rotate) {
 
 XS(_wrap_PDF_skew) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
+    float _arg1;
+    float _arg2;
     dXSARGS ;
 
     if (items != 3)
@@ -1233,12 +1240,12 @@ XS(_wrap_PDF_skew) {
 
 XS(_wrap_PDF_concat) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
-    float  _arg3;
-    float  _arg4;
-    float  _arg5;
-    float  _arg6;
+    float _arg1;
+    float _arg2;
+    float _arg3;
+    float _arg4;
+    float _arg5;
+    float _arg6;
     dXSARGS ;
 
     if (items != 7)
@@ -1263,10 +1270,64 @@ XS(_wrap_PDF_concat) {
     XSRETURN(0);
 }
 
+XS(_wrap_PDF_initgraphics) {
+    PDF * _arg0;
+    int argvi = 0;
+    dXSARGS ;
+
+    if (items != 1)
+        croak("Usage: PDF_initgraphics(p);");
+
+    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"PDFPtr")) {
+        croak("Type error in argument 1 of PDF_initgraphics. Expected PDFPtr.");
+        XSRETURN(1);
+    }
+
+    try {     PDF_initgraphics(_arg0);
+    }
+    else {
+	SWIG_exception(error_message);
+    }
+    XSRETURN(argvi);
+}
+
+XS(_wrap_PDF_setmatrix) {
+    PDF * _arg0;
+    float _arg1;
+    float _arg2;
+    float _arg3;
+    float _arg4;
+    float _arg5;
+    float _arg6;
+    int argvi = 0;
+    dXSARGS ;
+
+    if (items != 7)
+        croak("Usage: PDF_setmatrix(p,a,b,c,d,e,f);");
+
+    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"PDFPtr")) {
+        croak("Type error in argument 1 of PDF_setmatrix. Expected PDFPtr.");
+        XSRETURN(1);
+    }
+    _arg1 = (float ) SvNV(ST(1));
+    _arg2 = (float ) SvNV(ST(2));
+    _arg3 = (float ) SvNV(ST(3));
+    _arg4 = (float ) SvNV(ST(4));
+    _arg5 = (float ) SvNV(ST(5));
+    _arg6 = (float ) SvNV(ST(6));
+
+    try {     PDF_setmatrix(_arg0,_arg1,_arg2,_arg3,_arg4,_arg5,_arg6);
+    }
+    else {
+	SWIG_exception(error_message);
+    }
+    XSRETURN(argvi);
+}
+
 XS(_wrap_PDF_moveto) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
+    float _arg1;
+    float _arg2;
     dXSARGS ;
 
     if (items != 3)
@@ -1288,8 +1349,8 @@ XS(_wrap_PDF_moveto) {
 
 XS(_wrap_PDF_lineto) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
+    float _arg1;
+    float _arg2;
     dXSARGS ;
 
     if (items != 3)
@@ -1311,12 +1372,12 @@ XS(_wrap_PDF_lineto) {
 
 XS(_wrap_PDF_curveto) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
-    float  _arg3;
-    float  _arg4;
-    float  _arg5;
-    float  _arg6;
+    float _arg1;
+    float _arg2;
+    float _arg3;
+    float _arg4;
+    float _arg5;
+    float _arg6;
     dXSARGS ;
 
     if (items != 7)
@@ -1342,9 +1403,9 @@ XS(_wrap_PDF_curveto) {
 
 XS(_wrap_PDF_circle) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
-    float  _arg3;
+    float _arg1;
+    float _arg2;
+    float _arg3;
     dXSARGS ;
 
     if (items != 4)
@@ -1367,11 +1428,11 @@ XS(_wrap_PDF_circle) {
 
 XS(_wrap_PDF_arc) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
-    float  _arg3;
-    float  _arg4;
-    float  _arg5;
+    float _arg1;
+    float _arg2;
+    float _arg3;
+    float _arg4;
+    float _arg5;
     dXSARGS ;
 
     if (items != 6)
@@ -1394,12 +1455,43 @@ XS(_wrap_PDF_arc) {
     XSRETURN(0);
 }
 
+XS(_wrap_PDF_arcn) {
+    PDF * _arg0;
+    float _arg1;
+    float _arg2;
+    float _arg3;
+    float _arg4;
+    float _arg5;
+    int argvi = 0;
+    dXSARGS ;
+
+    if (items != 6)
+        croak("Usage: PDF_arcn(p,x,y,r,alpha,beta);");
+
+    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"PDFPtr")) {
+        croak("Type error in argument 1 of PDF_arcn. Expected PDFPtr.");
+        XSRETURN(1);
+    }
+    _arg1 = (float ) SvNV(ST(1));
+    _arg2 = (float ) SvNV(ST(2));
+    _arg3 = (float ) SvNV(ST(3));
+    _arg4 = (float ) SvNV(ST(4));
+    _arg5 = (float ) SvNV(ST(5));
+
+    try {     PDF_arcn(_arg0,_arg1,_arg2,_arg3,_arg4,_arg5);
+    }
+    else {
+	SWIG_exception(error_message);
+    }
+    XSRETURN(argvi);
+}
+
 XS(_wrap_PDF_rect) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
-    float  _arg3;
-    float  _arg4;
+    float _arg1;
+    float _arg2;
+    float _arg3;
+    float _arg4;
     dXSARGS ;
 
     if (items != 5)
@@ -1575,7 +1667,7 @@ XS(_wrap_PDF_clip) {
 
 XS(_wrap_PDF_setgray_fill) {
     PDF * _arg0;
-    float  _arg1;
+    float _arg1;
     dXSARGS ;
 
     if (items != 2)
@@ -1586,7 +1678,7 @@ XS(_wrap_PDF_setgray_fill) {
     }
     _arg1 = (float ) SvNV(ST(1));
 
-    try {     PDF_setgray_fill(_arg0,_arg1);
+    try {	PDF_setcolor(_arg0, "fill", "gray", _arg1, 0, 0, 0);
     }
     else {
 	SWIG_exception(error_message);
@@ -1596,7 +1688,7 @@ XS(_wrap_PDF_setgray_fill) {
 
 XS(_wrap_PDF_setgray_stroke) {
     PDF * _arg0;
-    float  _arg1;
+    float _arg1;
     dXSARGS ;
 
     if (items != 2)
@@ -1607,7 +1699,7 @@ XS(_wrap_PDF_setgray_stroke) {
     }
     _arg1 = (float ) SvNV(ST(1));
 
-    try {     PDF_setgray_stroke(_arg0,_arg1);
+    try {	PDF_setcolor(_arg0, "stroke", "gray", _arg1, 0, 0, 0);
     }
     else {
 	SWIG_exception(error_message);
@@ -1617,7 +1709,7 @@ XS(_wrap_PDF_setgray_stroke) {
 
 XS(_wrap_PDF_setgray) {
     PDF * _arg0;
-    float  _arg1;
+    float _arg1;
     dXSARGS ;
 
     if (items != 2)
@@ -1628,7 +1720,7 @@ XS(_wrap_PDF_setgray) {
     }
     _arg1 = (float ) SvNV(ST(1));
 
-    try {     PDF_setgray(_arg0,_arg1);
+    try {	PDF_setcolor(_arg0, "both", "gray", _arg1, 0, 0, 0);
     }
     else {
 	SWIG_exception(error_message);
@@ -1638,9 +1730,9 @@ XS(_wrap_PDF_setgray) {
 
 XS(_wrap_PDF_setrgbcolor_fill) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
-    float  _arg3;
+    float _arg1;
+    float _arg2;
+    float _arg3;
     dXSARGS ;
 
     if (items != 4)
@@ -1653,7 +1745,7 @@ XS(_wrap_PDF_setrgbcolor_fill) {
     _arg2 = (float ) SvNV(ST(2));
     _arg3 = (float ) SvNV(ST(3));
 
-    try {     PDF_setrgbcolor_fill(_arg0,_arg1,_arg2,_arg3);
+    try {	PDF_setcolor(_arg0, "fill", "rgb", _arg1, _arg2, _arg3, 0);
     }
     else {
 	SWIG_exception(error_message);
@@ -1663,9 +1755,9 @@ XS(_wrap_PDF_setrgbcolor_fill) {
 
 XS(_wrap_PDF_setrgbcolor_stroke) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
-    float  _arg3;
+    float _arg1;
+    float _arg2;
+    float _arg3;
     dXSARGS ;
 
     if (items != 4)
@@ -1678,7 +1770,7 @@ XS(_wrap_PDF_setrgbcolor_stroke) {
     _arg2 = (float ) SvNV(ST(2));
     _arg3 = (float ) SvNV(ST(3));
 
-    try {     PDF_setrgbcolor_stroke(_arg0,_arg1,_arg2,_arg3);
+    try {	PDF_setcolor(_arg0, "stroke", "rgb", _arg1, _arg2, _arg3, 0);
     }
     else {
 	SWIG_exception(error_message);
@@ -1688,9 +1780,9 @@ XS(_wrap_PDF_setrgbcolor_stroke) {
 
 XS(_wrap_PDF_setrgbcolor) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
-    float  _arg3;
+    float _arg1;
+    float _arg2;
+    float _arg3;
     dXSARGS ;
 
     if (items != 4)
@@ -1703,7 +1795,7 @@ XS(_wrap_PDF_setrgbcolor) {
     _arg2 = (float ) SvNV(ST(2));
     _arg3 = (float ) SvNV(ST(3));
 
-    try {     PDF_setrgbcolor(_arg0,_arg1,_arg2,_arg3);
+    try {	PDF_setcolor(_arg0, "both", "rgb", _arg1, _arg2, _arg3, 0);
     }
     else {
 	SWIG_exception(error_message);
@@ -1711,12 +1803,181 @@ XS(_wrap_PDF_setrgbcolor) {
     XSRETURN(0);
 }
 
+XS(_wrap_PDF_makespotcolor) {
+    int _result;
+    PDF * _arg0;
+    char * PDF_VOLATILE _arg1;
+    int PDF_VOLATILE _arg2;
+    int PDF_VOLATILE argvi = 0;
+    dXSARGS ;
+
+    if (items != 3)
+        croak("Usage: PDF_makespotcolor(p,spotname,len);");
+
+    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"PDFPtr")) {
+        croak("Type error in argument 1 of PDF_makespotcolor. Expected PDFPtr.");
+        XSRETURN(1);
+    }
+    _arg1 = (char *) SvPV(ST(1),PL_na);
+    _arg2 = (int )SvIV(ST(2));
+
+    try {     _result = (int )PDF_makespotcolor(_arg0,_arg1,_arg2);
+    }
+    else {
+	SWIG_exception(error_message);
+    }
+
+    ST(argvi) = sv_newmortal();
+    sv_setiv(ST(argvi++),(IV) _result);
+    XSRETURN(argvi);
+}
+
+XS(_wrap_PDF_setcolor) {
+    PDF * _arg0;
+    char * PDF_VOLATILE _arg1;
+    char * PDF_VOLATILE _arg2;
+    float _arg3;
+    float _arg4;
+    float _arg5;
+    float _arg6;
+    int argvi = 0;
+    dXSARGS ;
+
+    if (items != 7)
+        croak("Usage: PDF_setcolor(p,type,colorspace,c1,c2,c3,c4);");
+
+    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"PDFPtr")) {
+        croak("Type error in argument 1 of PDF_setcolor. Expected PDFPtr.");
+        XSRETURN(1);
+    }
+    _arg1 = (char *) SvPV(ST(1),PL_na);
+    _arg2 = (char *) SvPV(ST(2),PL_na);
+    _arg3 = (float ) SvNV(ST(3));
+    _arg4 = (float ) SvNV(ST(4));
+    _arg5 = (float ) SvNV(ST(5));
+    _arg6 = (float ) SvNV(ST(6));
+
+    try {     PDF_setcolor(_arg0,_arg1,_arg2,_arg3,_arg4,_arg5,_arg6);
+    }
+    else {
+	SWIG_exception(error_message);
+    }
+
+    XSRETURN(argvi);
+}
+
+XS(_wrap_PDF_begin_pattern) {
+    int _result;
+    PDF * _arg0;
+    float _arg1;
+    float _arg2;
+    float _arg3;
+    float _arg4;
+    int PDF_VOLATILE _arg5;
+    int PDF_VOLATILE argvi = 0;
+    dXSARGS ;
+
+    if (items != 6)
+        croak("Usage: PDF_begin_pattern(p,width,height,xstep,ystep,painttype);");
+    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"PDFPtr")) {
+        croak("Type error in argument 1 of PDF_begin_pattern. Expected PDFPtr.");
+        XSRETURN(1);
+    }
+    _arg1 = (float ) SvNV(ST(1));
+    _arg2 = (float ) SvNV(ST(2));
+    _arg3 = (float ) SvNV(ST(3));
+    _arg4 = (float ) SvNV(ST(4));
+    _arg5 = (int )SvIV(ST(5));
+
+    try {     _result = (int )PDF_begin_pattern(_arg0,_arg1,_arg2,_arg3,_arg4,_arg5);
+    }
+    else {
+	SWIG_exception(error_message);
+    }
+
+    ST(argvi) = sv_newmortal();
+    sv_setiv(ST(argvi++),(IV) _result);
+    XSRETURN(argvi);
+}
+
+XS(_wrap_PDF_end_pattern) {
+    PDF * _arg0;
+    int argvi = 0;
+    dXSARGS ;
+
+    if (items != 1)
+        croak("Usage: PDF_end_pattern(p);");
+
+    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"PDFPtr")) {
+        croak("Type error in argument 1 of PDF_end_pattern. Expected PDFPtr.");
+        XSRETURN(1);
+    }
+
+    try {     PDF_end_pattern(_arg0);
+    }
+    else {
+	SWIG_exception(error_message);
+    }
+    XSRETURN(argvi);
+}
+
+XS(_wrap_PDF_begin_template) {
+    int _result;
+    PDF * _arg0;
+    float _arg1;
+    float _arg2;
+    int PDF_VOLATILE argvi = 0;
+    dXSARGS ;
+
+    if (items != 3)
+        croak("Usage: PDF_begin_template(p,width,height);");
+
+    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"PDFPtr")) {
+        croak("Type error in argument 1 of PDF_begin_template. Expected PDFPtr.");
+        XSRETURN(1);
+    }
+    _arg1 = (float ) SvNV(ST(1));
+    _arg2 = (float ) SvNV(ST(2));
+
+    try {     _result = (int )PDF_begin_template(_arg0,_arg1,_arg2);
+    }
+    else {
+	SWIG_exception(error_message);
+    }
+
+    ST(argvi) = sv_newmortal();
+    sv_setiv(ST(argvi++),(IV) _result);
+    XSRETURN(argvi);
+}
+
+XS(_wrap_PDF_end_template) {
+    PDF * _arg0;
+    int argvi = 0;
+    dXSARGS ;
+
+    if (items != 1)
+        croak("Usage: PDF_end_template(p);");
+
+    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"PDFPtr")) {
+        croak("Type error in argument 1 of PDF_end_template. Expected PDFPtr.");
+        XSRETURN(1);
+    }
+
+    try {     PDF_end_template(_arg0);
+    }
+    else {
+	SWIG_exception(error_message);
+    }
+
+    XSRETURN(argvi);
+}
+
 XS(_wrap_PDF_place_image) {
     PDF * _arg0;
-    int  _arg1;
-    float  _arg2;
-    float  _arg3;
-    float  _arg4;
+    int PDF_VOLATILE _arg1;
+    float _arg2;
+    float _arg3;
+    float _arg4;
     dXSARGS ;
 
     if (items != 5)
@@ -1739,18 +2000,18 @@ XS(_wrap_PDF_place_image) {
 }
 
 XS(_wrap_PDF_open_image) {
-    int  _result;
+    int _result;
     PDF * _arg0;
-    char * _arg1;
-    char * _arg2;
-    char * _arg3;
-    long  _arg4;
-    int  _arg5;
-    int  _arg6;
-    int  _arg7;
-    int  _arg8;
-    char * _arg9;
-    int argvi = 0;
+    char * PDF_VOLATILE _arg1;
+    char * PDF_VOLATILE _arg2;
+    char * PDF_VOLATILE _arg3;
+    long  PDF_VOLATILE _arg4;
+    int PDF_VOLATILE _arg5;
+    int PDF_VOLATILE _arg6;
+    int PDF_VOLATILE _arg7;
+    int PDF_VOLATILE _arg8;
+    char * PDF_VOLATILE _arg9;
+    int PDF_VOLATILE argvi = 0;
     dXSARGS ;
 
     if (items != 10)
@@ -1781,7 +2042,7 @@ XS(_wrap_PDF_open_image) {
 
 XS(_wrap_PDF_close_image) {
     PDF * _arg0;
-    int  _arg1;
+    int PDF_VOLATILE _arg1;
     dXSARGS ;
 
     if (items != 2)
@@ -1800,16 +2061,39 @@ XS(_wrap_PDF_close_image) {
     XSRETURN(0);
 }
 
-XS(_wrap_PDF_open_image_file) {
-    int  _result;
+XS(_wrap_PDF_add_thumbnail) {
     PDF * _arg0;
-    char * _arg1;
-    char * _arg2;
-    char * _arg3;
-    int  _arg4;
+    int PDF_VOLATILE _arg1;
     int argvi = 0;
     dXSARGS ;
 
+    if (items != 2)
+        croak("Usage: PDF_add_thumbnail(p,image);");
+
+    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"PDFPtr")) {
+        croak("Type error in argument 1 of PDF_add_thumbnail. Expected PDFPtr.");
+        XSRETURN(1);
+    }
+    _arg1 = (int )SvIV(ST(1));
+
+    try {     PDF_add_thumbnail(_arg0,_arg1);
+    }
+    else {
+	SWIG_exception(error_message);
+    }
+    XSRETURN(argvi);
+}
+
+XS(_wrap_PDF_open_image_file) {
+    int _result;
+    PDF * _arg0;
+    char * PDF_VOLATILE _arg1;
+    char * PDF_VOLATILE _arg2;
+    char * PDF_VOLATILE _arg3;
+    int PDF_VOLATILE _arg4;
+    int PDF_VOLATILE argvi = 0;
+    dXSARGS ;
+
     if (items != 5)
         croak("Usage: PDF_open_image_file(p,type,filename,stringparam,intparam);");
     if (SWIG_GetPtr(ST(0),(void **) &_arg0,"PDFPtr")) {
@@ -1832,15 +2116,15 @@ XS(_wrap_PDF_open_image_file) {
 }
 
 XS(_wrap_PDF_open_CCITT) {
-    int  _result;
+    int _result;
     PDF * _arg0;
-    char * _arg1;
-    int  _arg2;
-    int  _arg3;
-    int  _arg4;
-    int  _arg5;
-    int  _arg6;
-    int argvi = 0;
+    char * PDF_VOLATILE _arg1;
+    int PDF_VOLATILE _arg2;
+    int PDF_VOLATILE _arg3;
+    int PDF_VOLATILE _arg4;
+    int PDF_VOLATILE _arg5;
+    int PDF_VOLATILE _arg6;
+    int PDF_VOLATILE argvi = 0;
     dXSARGS ;
 
     if (items != 7)
@@ -1867,12 +2151,12 @@ XS(_wrap_PDF_open_CCITT) {
 }
 
 XS(_wrap_PDF_add_bookmark) {
-    int  _result;
+    int _result;
     PDF * _arg0;
-    char * _arg1;
-    int  _arg2;
-    int  _arg3;
-    int argvi = 0;
+    char * PDF_VOLATILE _arg1;
+    int PDF_VOLATILE _arg2;
+    int PDF_VOLATILE _arg3;
+    int PDF_VOLATILE argvi = 0;
     dXSARGS ;
 
     if (items != 4)
@@ -1897,8 +2181,8 @@ XS(_wrap_PDF_add_bookmark) {
 
 XS(_wrap_PDF_set_info) {
     PDF * _arg0;
-    char * _arg1;
-    char * _arg2;
+    char * PDF_VOLATILE _arg1;
+    char * PDF_VOLATILE _arg2;
     dXSARGS ;
 
     if (items != 3)
@@ -1920,15 +2204,15 @@ XS(_wrap_PDF_set_info) {
 
 XS(_wrap_PDF_attach_file) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
-    float  _arg3;
-    float  _arg4;
-    char * _arg5;
-    char * _arg6;
-    char * _arg7;
-    char * _arg8;
-    char * _arg9;
+    float _arg1;
+    float _arg2;
+    float _arg3;
+    float _arg4;
+    char * PDF_VOLATILE _arg5;
+    char * PDF_VOLATILE _arg6;
+    char * PDF_VOLATILE _arg7;
+    char * PDF_VOLATILE _arg8;
+    char * PDF_VOLATILE _arg9;
     dXSARGS ;
 
     if (items != 10)
@@ -1957,14 +2241,14 @@ XS(_wrap_PDF_attach_file) {
 
 XS(_wrap_PDF_add_note) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
-    float  _arg3;
-    float  _arg4;
-    char * _arg5;
-    char * _arg6;
-    char * _arg7;
-    int  _arg8;
+    float _arg1;
+    float _arg2;
+    float _arg3;
+    float _arg4;
+    char * PDF_VOLATILE _arg5;
+    char * PDF_VOLATILE _arg6;
+    char * PDF_VOLATILE _arg7;
+    int PDF_VOLATILE _arg8;
     dXSARGS ;
 
     if (items != 9)
@@ -1992,13 +2276,13 @@ XS(_wrap_PDF_add_note) {
 
 XS(_wrap_PDF_add_pdflink) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
-    float  _arg3;
-    float  _arg4;
-    char * _arg5;
-    int  _arg6;
-    char * _arg7;
+    float _arg1;
+    float _arg2;
+    float _arg3;
+    float _arg4;
+    char * PDF_VOLATILE _arg5;
+    int PDF_VOLATILE _arg6;
+    char * PDF_VOLATILE _arg7;
     dXSARGS ;
 
     if (items != 8)
@@ -2025,11 +2309,11 @@ XS(_wrap_PDF_add_pdflink) {
 
 XS(_wrap_PDF_add_launchlink) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
-    float  _arg3;
-    float  _arg4;
-    char * _arg5;
+    float _arg1;
+    float _arg2;
+    float _arg3;
+    float _arg4;
+    char * PDF_VOLATILE _arg5;
     dXSARGS ;
 
     if (items != 6)
@@ -2054,12 +2338,12 @@ XS(_wrap_PDF_add_launchlink) {
 
 XS(_wrap_PDF_add_locallink) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
-    float  _arg3;
-    float  _arg4;
-    int  _arg5;
-    char * _arg6;
+    float _arg1;
+    float _arg2;
+    float _arg3;
+    float _arg4;
+    int PDF_VOLATILE _arg5;
+    char * PDF_VOLATILE _arg6;
     dXSARGS ;
 
     if (items != 7)
@@ -2085,11 +2369,11 @@ XS(_wrap_PDF_add_locallink) {
 
 XS(_wrap_PDF_add_weblink) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
-    float  _arg3;
-    float  _arg4;
-    char * _arg5;
+    float _arg1;
+    float _arg2;
+    float _arg3;
+    float _arg4;
+    char * PDF_VOLATILE _arg5;
     dXSARGS ;
 
     if (items != 6)
@@ -2114,8 +2398,8 @@ XS(_wrap_PDF_add_weblink) {
 
 XS(_wrap_PDF_set_border_style) {
     PDF * _arg0;
-    char * _arg1;
-    float  _arg2;
+    char * PDF_VOLATILE _arg1;
+    float _arg2;
     dXSARGS ;
 
     if (items != 3)
@@ -2137,9 +2421,9 @@ XS(_wrap_PDF_set_border_style) {
 
 XS(_wrap_PDF_set_border_color) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
-    float  _arg3;
+    float _arg1;
+    float _arg2;
+    float _arg3;
     dXSARGS ;
 
     if (items != 4)
@@ -2162,8 +2446,8 @@ XS(_wrap_PDF_set_border_color) {
 
 XS(_wrap_PDF_set_border_dash) {
     PDF * _arg0;
-    float  _arg1;
-    float  _arg2;
+    float _arg1;
+    float _arg2;
     dXSARGS ;
 
     if (items != 3)
@@ -2183,8 +2467,213 @@ XS(_wrap_PDF_set_border_dash) {
     XSRETURN(0);
 }
 
+XS(_wrap_PDF_open_pdi) {
+    int _result;
+    PDF * _arg0;
+    char * PDF_VOLATILE _arg1;
+    char * PDF_VOLATILE _arg2;
+    int PDF_VOLATILE _arg3;
+    int PDF_VOLATILE argvi = 0;
+    dXSARGS ;
+
+    if (items != 4) 
+        croak("Usage: PDF_open_pdi(p,filename,stringparam,intparam);");
+
+    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"PDFPtr")) {
+        croak("Type error in argument 1 of PDF_open_pdi. Expected PDFPtr.");
+        XSRETURN(1);
+    }
+    _arg1 = (char *) SvPV(ST(1),PL_na);
+    _arg2 = (char *) SvPV(ST(2),PL_na);
+    _arg3 = (int )SvIV(ST(3));
+
+    try {     _result = (int )PDF_open_pdi(_arg0,_arg1,_arg2,_arg3);
+    }
+    else {
+	SWIG_exception(error_message);
+    }
+
+    ST(argvi) = sv_newmortal();
+    sv_setiv(ST(argvi++),(IV) _result);
+    XSRETURN(argvi);
+}
+
+XS(_wrap_PDF_close_pdi) {
+    PDF * _arg0;
+    int PDF_VOLATILE _arg1;
+    int argvi = 0;
+    dXSARGS ;
+
+    if (items != 2)
+        croak("Usage: PDF_close_pdi(p,doc);");
+
+    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"PDFPtr")) {
+        croak("Type error in argument 1 of PDF_close_pdi. Expected PDFPtr.");
+        XSRETURN(1);
+    }
+    _arg1 = (int )SvIV(ST(1));
+
+    try {     PDF_close_pdi(_arg0,_arg1);
+    }
+    else {
+	SWIG_exception(error_message);
+    }
+    XSRETURN(argvi);
+}
+
+XS(_wrap_PDF_open_pdi_page) {
+    int _result;
+    PDF * _arg0;
+    int PDF_VOLATILE _arg1;
+    int PDF_VOLATILE _arg2;
+    char * PDF_VOLATILE _arg3;
+    int PDF_VOLATILE argvi = 0;
+    dXSARGS ;
+
+    if (items != 4)
+        croak("Usage: PDF_open_pdi_page(p,doc,page,label);");
+
+    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"PDFPtr")) {
+        croak("Type error in argument 1 of PDF_open_pdi_page. Expected PDFPtr.");
+        XSRETURN(1);
+    }
+    _arg1 = (int )SvIV(ST(1));
+    _arg2 = (int )SvIV(ST(2));
+    _arg3 = (char *) SvPV(ST(3),PL_na);
+
+    try {     _result = (int )PDF_open_pdi_page(_arg0,_arg1,_arg2,_arg3);
+    }
+    else {
+	SWIG_exception(error_message);
+    }
+
+    ST(argvi) = sv_newmortal();
+    sv_setiv(ST(argvi++),(IV) _result);
+    XSRETURN(argvi);
+}
+
+XS(_wrap_PDF_close_pdi_page) {
+    PDF * _arg0;
+    int PDF_VOLATILE _arg1;
+    int argvi = 0;
+    dXSARGS ;
+
+    if (items != 2)
+        croak("Usage: PDF_close_pdi_page(p,page);");
+
+    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"PDFPtr")) {
+        croak("Type error in argument 1 of PDF_close_pdi_page. Expected PDFPtr.");
+        XSRETURN(1);
+    }
+    _arg1 = (int )SvIV(ST(1));
+
+    try {     PDF_close_pdi_page(_arg0,_arg1);
+    }
+    else {
+	SWIG_exception(error_message);
+    }
+    XSRETURN(argvi);
+}
+
+XS(_wrap_PDF_place_pdi_page) {
+    PDF * _arg0;
+    int PDF_VOLATILE _arg1;
+    float _arg2;
+    float _arg3;
+    float _arg4;
+    float _arg5;
+    dXSARGS ;
+
+    if (items != 6)
+        croak("Usage: PDF_place_pdi_page(p,image,x,y,sx,sy);");
+    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"PDFPtr")) {
+        croak("Type error in argument 1 of PDF_place_pdi_page. Expected PDFPtr.");
+        XSRETURN(1);
+    }
+    _arg1 = (int )SvIV(ST(1));
+    _arg2 = (float ) SvNV(ST(2));
+    _arg3 = (float ) SvNV(ST(3));
+    _arg4 = (float ) SvNV(ST(4));
+    _arg5 = (float ) SvNV(ST(5));
+
+    try {     PDF_place_pdi_page(_arg0,_arg1,_arg2,_arg3,_arg4,_arg5);
+    }
+    else {
+	SWIG_exception(error_message);
+    }
+    XSRETURN(0);
+}
+
+XS(_wrap_PDF_get_pdi_parameter) {
+    char * _result;
+    PDF * _arg0;
+    char * PDF_VOLATILE _arg1;
+    int PDF_VOLATILE _arg2;
+    int PDF_VOLATILE _arg3;
+    int PDF_VOLATILE _arg4;
+    int len;
+    dXSARGS ;
+
+    if (items != 5)
+        croak("Usage: PDF_get_pdi_parameter(p,key,doc,page,index);");
+
+    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"PDFPtr")) {
+        croak("Type error in argument 1 of PDF_get_pdi_parameter. Expected PDFPtr.");
+        XSRETURN(1);
+    }
+    _arg1 = (char *) SvPV(ST(1),PL_na);
+    _arg2 = (int )SvIV(ST(2));
+    _arg3 = (int )SvIV(ST(3));
+    _arg4 = (int )SvIV(ST(4));
+
+    try {     _result = (char *)PDF_get_pdi_parameter(_arg0,_arg1,_arg2,_arg3,_arg4, &len);
+    }
+    else {
+	SWIG_exception(error_message);
+    }
+
+    ST(0) = sv_newmortal();
+    sv_setpvn((SV*)ST(0), (char *) _result, len);
+    XSRETURN(1);
+}
+
+XS(_wrap_PDF_get_pdi_value) {
+    float _result;
+    PDF * _arg0;
+    char * PDF_VOLATILE _arg1;
+    int PDF_VOLATILE _arg2;
+    int PDF_VOLATILE _arg3;
+    int PDF_VOLATILE _arg4;
+    int PDF_VOLATILE argvi = 0;
+    dXSARGS ;
+
+    if (items != 5)
+        croak("Usage: PDF_get_pdi_value(p,key,doc,page,index);");
+
+    if (SWIG_GetPtr(ST(0),(void **) &_arg0,"PDFPtr")) {
+        croak("Type error in argument 1 of PDF_get_pdi_value. Expected PDFPtr.");
+        XSRETURN(1);
+    }
+    _arg1 = (char *) SvPV(ST(1),PL_na);
+    _arg2 = (int )SvIV(ST(2));
+    _arg3 = (int )SvIV(ST(3));
+    _arg4 = (int )SvIV(ST(4));
+
+    try {     _result = (float )PDF_get_pdi_value(_arg0,_arg1,_arg2,_arg3,_arg4);
+    }
+    else {
+	SWIG_exception(error_message);
+    }
+
+    ST(argvi) = sv_newmortal();
+    sv_setnv(ST(argvi++), (double) _result);
+    XSRETURN(argvi);
+}
+
 XS(_wrap_perl5_pdflib_var_init) {
     dXSARGS;
+    (void) items;
+
     XSRETURN(1);
 }
 #ifdef __cplusplus
@@ -2195,6 +2684,8 @@ XS(boot_pdflib_pl) {
     char *file = __FILE__;
     newXS("pdflibc::var_pdflib_init", _wrap_perl5_pdflib_var_init, file);
 
+    (void) items;
+
     /* Boot the PDFlib core */
     PDF_boot();
 
@@ -2268,6 +2759,24 @@ XS(boot_pdflib_pl) {
     newXS("pdflibc::PDF_set_border_style", _wrap_PDF_set_border_style, file);
     newXS("pdflibc::PDF_set_border_color", _wrap_PDF_set_border_color, file);
     newXS("pdflibc::PDF_set_border_dash", _wrap_PDF_set_border_dash, file);
+    newXS("pdflibc::PDF_open_pdi", _wrap_PDF_open_pdi, file);
+    newXS("pdflibc::PDF_close_pdi", _wrap_PDF_close_pdi, file);
+    newXS("pdflibc::PDF_open_pdi_page", _wrap_PDF_open_pdi_page, file);
+    newXS("pdflibc::PDF_close_pdi_page", _wrap_PDF_close_pdi_page, file);
+    newXS("pdflibc::PDF_place_pdi_page", _wrap_PDF_place_pdi_page, file);
+    newXS("pdflibc::PDF_get_pdi_parameter", _wrap_PDF_get_pdi_parameter, file);
+    newXS("pdflibc::PDF_get_pdi_value", _wrap_PDF_get_pdi_value, file);
+
+    newXS("pdflibc::PDF_makespotcolor", _wrap_PDF_makespotcolor, file);
+    newXS("pdflibc::PDF_setcolor", _wrap_PDF_setcolor, file);
+    newXS("pdflibc::PDF_begin_pattern", _wrap_PDF_begin_pattern, file);
+    newXS("pdflibc::PDF_end_pattern", _wrap_PDF_end_pattern, file);
+    newXS("pdflibc::PDF_begin_template", _wrap_PDF_begin_template, file);
+    newXS("pdflibc::PDF_end_template", _wrap_PDF_end_template, file);
+    newXS("pdflibc::PDF_initgraphics", _wrap_PDF_initgraphics, file);
+    newXS("pdflibc::PDF_setmatrix", _wrap_PDF_setmatrix, file);
+    newXS("pdflibc::PDF_arcn", _wrap_PDF_arcn, file);
+    newXS("pdflibc::PDF_add_thumbnail", _wrap_PDF_add_thumbnail, file);
 /*
  * These are the pointer type-equivalency mappings. 
  * (Used by the SWIG pointer type-checker).
